return (

    # Evènements d'éditeurs
    'cursor_set_last' => 'Text::Editor::Easy',
    'motion_last'     => 'Text::Editor::Easy',
    'on_top_last'     => 'Text::Editor::Easy',
    'insert_last'     => 'Text::Editor::Easy',
    'change_last'     => 'Text::Editor::Easy',
	'clic_last'     => 'Text::Editor::Easy',

    # Evènements de zones
    'on_top_editor_change' => 'Text::Editor::Easy::Zone',
	'on_editor_destroy' => 'Text::Editor::Easy::Zone',
	'on_new_editor' => 'Text::Editor::Easy::Zone',
);

=head1 COPYRIGHT & LICENSE

Copyright 2008 Sebastien Grommier, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

