#
# What about an interactive test of perl regexp ?
#
# Search implementation is not yet finished, but here is an other possibility
# of what a perl Editor could be used to. Interactive tutorials are a more
# friendly way to learn than a text file. If there are more funny ways to learn
# perl, maybe there will be more perl programmers and less wasted time
# to learn other inefficient langages !
# 
# As usual, press F5 to insert search code in the macro panel
# You can then press the arrows "Up" or "Down" to navigate into
# the different expressions to be searched.
# You can also put your own expression in the second line of the macro
# instructions.
# 
# You'll notice that, for the $exp variable, you can put a string for an
# exact match (not a regular expression search) or a regexp with the perl
# syntax : qr/regexp/modifiers
#